#include "construction.hh"

MyDetectorConstruction::MyDetectorConstruction(){

    nCols = 10;
    nRows = 10;
    
    fMessenger = new G4GenericMessenger(this, "/detector/", "Detector Construction");
    fMessenger->DeclareProperty("nCols", nCols, "Number of Colums");
    fMessenger->DeclareProperty("nRows", nRows, "Number of Rows");

    DefineMaterial();
}

MyDetectorConstruction::~MyDetectorConstruction(){

}

void MyDetectorConstruction::DefineMaterial(){


    G4NistManager *nist  = G4NistManager::Instance();

    worldMat = nist->FindOrBuildMaterial("G4_AIR");

    elCd = new G4Element("Cadmium", "Cd", 48., 112.411*g/mole);
    elZn = new G4Element("Zinc", "Zn", 30., 65.38*g/mole);
    elTe = new G4Element("Tellurium", "Te", 52., 127.60*g/mole);
    elSe = new G4Element("Selenium", "Se", 34., 78.971*g/mole);

    double density = 5.8*g/cm3; // Assume or adjust based on literature/experimental data
    CdZnTeSe = new G4Material("CdZnTeSe", density, 4); // 4 components

    CdZnTeSe->AddElement(elCd, 90);  // Proportion of Cd
    CdZnTeSe->AddElement(elZn, 10);  // Proportion of Zn
    CdZnTeSe->AddElement(elTe, 98); // Proportion of Te
    CdZnTeSe->AddElement(elSe, 2); // Proportion of Se*/

    tungsten = nist->FindOrBuildMaterial("G4_W");
    beryllium = nist->FindOrBuildMaterial("G4_Be");

}

G4VPhysicalVolume *MyDetectorConstruction::Construct(){
        
    G4double xWorld = 0.1*m;
    G4double yWorld = 0.1*m;
    G4double zWorld = 0.1*m;
    solidWorld = new G4Box("solidWorld", xWorld,yWorld,zWorld);
    logicWorld = new G4LogicalVolume(solidWorld,worldMat,"logicWorld");
    pyhsworld = new G4PVPlacement(0, G4ThreeVector(0.,0.,0.), logicWorld,"physWorld", 0, false, 0, true);

    G4VisAttributes* visAttributes = new G4VisAttributes(G4Colour::Red());

    // Define W tube
    solidTube = new G4Tubs("Tube", 0.5*cm, 2.2*cm, 2.0*cm, 0., 2*CLHEP::pi);
    logicTube = new G4LogicalVolume(solidTube,tungsten,"logicTube");
    physTube = new G4PVPlacement(0,G4ThreeVector(0,0,0), logicTube,"PhysTube",
                                                    logicWorld, false,0, true);
    logicTube->SetVisAttributes(visAttributes);

    // EndCap Right
    solidEndcap = new G4Tubs("Endcap", 0*cm, 2.2*cm, 1.0*cm, 0., 2*CLHEP::pi);
    logicEndcap = new G4LogicalVolume(solidEndcap, tungsten, "logicEndcap");
    G4ThreeVector endcapPosition(0, 0, -3*cm);
    physEndcap = new G4PVPlacement(0, endcapPosition, logicEndcap, "PhysEndcap", logicWorld, false, 0, true);

    // Be Window
    solidBeCylinder = new G4Tubs("BeCylinder", 0.0*cm, 2.2*cm, 0.05*cm, 0., 2*CLHEP::pi);
    logicBeCylinder = new G4LogicalVolume(solidBeCylinder, beryllium, "logicBeCylinder");
    G4ThreeVector beCylinderPosition(0, 0, 2.05*cm); // Adjust the coordinates as needed
    physBeCylinder = new G4PVPlacement(0, beCylinderPosition, logicBeCylinder, "PhysBeCylinder", logicWorld, false, 0, true);

    solidDetector = new G4Box("solidDetector", 1*cm, 1*cm, 1*cm);
    logicDetector = new G4LogicalVolume(solidDetector, CdZnTeSe, "logicDetector");
    physDetector = new G4PVPlacement(0,G4ThreeVector(0,0,0.05*m), logicDetector, "physDetector", logicWorld, false, 0, true);

    fScoringVolume = logicDetector;

    return pyhsworld;
}

void MyDetectorConstruction::ConstructSDandField(){
    MySensitiveDetector *sensDet = new MySensitiveDetector("SensitiveDetector");
    logicDetector->SetSensitiveDetector(sensDet);
}