#ifndef CONSTRUCTION_HH
#define CONSTRUCTION_HH

#include "G4SystemOfUnits.hh"
#include "G4Box.hh"
#include "G4NistManager.hh"
#include "G4PVPlacement.hh"
#include "G4VPhysicalVolume.hh"
#include "G4LogicalVolume.hh"
#include "G4VUserDetectorConstruction.hh"
#include "G4GenericMessenger.hh"
#include "G4Tubs.hh"
#include "G4VisAttributes.hh"
#include "detector.hh"

class MyDetectorConstruction : public G4VUserDetectorConstruction{
public:
    MyDetectorConstruction();
    ~MyDetectorConstruction();
    virtual G4VPhysicalVolume *Construct();

    G4LogicalVolume *GetScoringVolume() const {return fScoringVolume;}

    G4LogicalVolume* GetLogicTube() const { return logicTube; }
    G4LogicalVolume* GetLogicEndcap() const { return logicEndcap; }
    G4LogicalVolume* GetLogicBeCylinder() const { return logicBeCylinder; }
private:
    
    virtual void ConstructSDandField();

    G4int nCols;
    G4int nRows;

    G4Box *solidWorld, *solidDetector;
    G4Tubs *solidTube, *solidEndcap, *solidBeCylinder;
    G4LogicalVolume *logicDetector, *logicWorld, *logicTube, *logicEndcap, *logicBeCylinder;
    G4VPhysicalVolume *physDetector, *pyhsworld, *physTube, *physEndcap, *physBeCylinder;

    G4GenericMessenger *fMessenger;

    G4Material  *worldMat, *CdZnTeSe, *tungsten, *beryllium;

    G4Element *elCd, *elZn, *elTe, *elSe;

    void DefineMaterial();

    G4LogicalVolume *fScoringVolume;
};

#endif